% Copyright (C) Paweł Płazieński 2009
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 2, or (at your option) any
% later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

get_line(Text) :-
	get_line(Text,[]).

get_line(Text,Completions) :-
	get_line(Text,Completions,'-.> ').

get_line(Text,Completions,Prompt) :-
	write(Prompt),
	get_line([],ReverseString,Completions,Prompt),
	reverse(ReverseString,String),
	string_to_list(Text,String).

get_line(Got,ReverseString,Completions,Prompt) :-
	get_single_char(B),
% 	write(B),
 (  (B = 13 ; B = 10),
 		nl, ttyflush,
		!,
		Got = ReverseString
	;  B = 9,
		reverse(Got,ReverseGot),
		string_to_atom(ReverseGot,Begin),
		find_begin(Completions,Begin,Begins),
		( Begins = [],
			nl, write('< - No completions'),
			nl, write(Prompt),
			(  Begin = []
			;  not(Begin = []),
				write(Begin)
			),
			get_line(Got,ReverseString,Completions,Prompt)
		; Begins = [Single],
			nl, write(Prompt), write(Single),
			string_to_atom(S,Single),
			string_to_list(S,G),
			reverse(G,ReverseG),
			get_line(ReverseG,ReverseString,Completions,Prompt)
		; Begins = [_|Tail], not(Tail = []),
			nl, write_list(Begins), write(Prompt),
			(  Begin = []
			;  not(Begin = []),
				write(Begin)
			),
			get_line(Got,ReverseString,Completions,Prompt)
		)
	;  B = 127,
			(  Got = [_|G],
 				put_byte(8),
 				put_byte(32),
 				put_byte(8),
				get_line(G,ReverseString,Completions,Prompt)
			;  Got = [],
				get_line(Got,ReverseString,Completions,Prompt)
			)
	;  not(B = 13 ; B = 10 ; B = 127; B = 9),
		char_code(C,B),
 		put_char(C),
		G = [B|Got],
		get_line(G,ReverseString,Completions,Prompt)
	).

empty_line(X) :-
	string_to_atom(X,'').

find_begin(Completions,[],Completions).

find_begin(Completions,Begin,Begins) :-
	bagof(X, (member(X,Completions), begins(Begin,X)), Begins), !.

find_begin(_,_,[]).

begins(Begin,String) :-
	string_concat(Begin,_,String).

write_list([]).
write_list([H|T]) :-
	write('<.- : '), write(H), nl,
	write_list(T).
